"""
    quadrature(f::Function, a::Real, b::Real) -> Float64 

Calculates definite integral of the function `f` in interval <`a`, `b`> using the Gaussian three point quadrature rule.

# Arguments
- `f::Function`: function to be integrated,
- `a::Real`: left-bound of the interval to be integrated on,
- `b::Real`: right-bound of the interval to be integrated on,

# Keywords

# Returns
- `Float64`: definite integral of the function `f` on interval <`a`, `b`>. 

# Throws
"""
function quadrature(f::Function, a::Real, b::Real)
    eps_f = 50*eps(Float64)
    eps_n = 1.0

    r = @SVector [-sqrt(3/5), 0.0, sqrt(3/5)]
    w = @SVector [5/9, 8/9, 5/9]

    n = 5

    so = 0.0
    sn = 0.0

    while eps_n > eps_f
        sn = 0.0
        x = range(a, stop=b, length=n)
        t1 = convert(Float64,x.step)/2
        for i in 1:n-1
            t2 = (x[i+1]+x[i])/2
            for i in eachindex(w)
                sn += t1*w[i]*f(t1*r[i]+t2)
            end
        end
        eps_n = abs(sn-so)
        so = sn
        n += 1
    end
    return so
end

"""
    curve_length(f::Function, a::Real, b::Real) -> Float64

Calculates the length of the curve given by `f`, where `f` is function of one parameter, on interval <`a`, `b`> of said parameter.

# Arguments
- `f::Function`: function of the curve to be measured,
- `a::Real`: left-bound of the interval to be integrated on,
- `b::Real`: right-bound of the interval to be integrated on,

# Keywords

# Returns
- `Float64`: length of the curve defined by the `f` on the interval <`a`, `b`>. 

# Throws
"""
function curve_length(f::Function, a::Real, b::Real)
    df = x-> ForwardDiff.derivative(f,x)
    dr = x-> sqrt(1+df(x)^2)
    l = quadrature(dr,a,b)
    return l
end

"""
    curve_fraction(f::Function,a::Real,b::Real,r::Real) -> Float64

Retunrs the position of the fraction of the length from the interval <`a`, `b`>, where the length of the curve is given by one parameter function `f`.

# Arguments
- `f::Function`: function of the curve,
- `a::Real`: left-bound of the interval to be integrated on,
- `b::Real`: right-bound of the interval to be integrated on,
- `r::Real`: fraction of the length of the curve given by `f` from the left,

# Keywords

# Returns
- `Float64`: position of the fraction of the length from the interval <`a`, `b`>. 

# Throws
"""
function curve_fraction(f::Function,a::Real,b::Real,r::Real)
    eps_f = 50*eps(Float64)
    eps_n = 1.0

    df = x -> ForwardDiff.derivative(f,x)
    dr = x -> sqrt(1.0 + df(x)^2)

    l = quadrature(dr,a,b)
    lf = l*r

    xo = (a+b)/2
    xn = 0.0
    while eps_n > eps_f
        xn = xo - (quadrature(dr,a,xo) - lf)/dr(xo)
        eps_n = abs(xo-xn)
        xo = xn
    end

    return xo
end
